@@ -1,10 +1,9 @@
 /****************************************************************/
-/* BGP-4 Protocol Viewer                                        */
 /*    Programmed By K.Kondo IIJ 2000/01                         */
 /*                                                              */
 /* BGPView Shell Commands                                       */
 /****************************************************************/
-/* bvcommand.c,v 1.38 2001/04/03 09:25:01 kuniaki Exp */
+/* bvcommand.c,v 1.40 2001/07/30 08:53:49 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 - 2001 Internet Initiative Japan Inc.
@@ -881,7 +880,11 @@ int pr_routesum(int clno, R_LIST *rlist, net_ulong neighbor)
 #ifdef RTHISTORY
 		/*** Display Update History ***/
 		if ((rinfo->history != NULL) && (detail == true)){
-			bvs_write(clno, "    == Prefix Update History ==\r\n");
+			pcl = rinfo->firstupdate.tv_sec;
+			tm = localtime(&pcl);
+			strftime(prline, 128,
+	 "    == Prefix Update History (First %Y/%m/%d %H:%M:%S) ==\r\n", tm);
+			bvs_write(clno, prline);
 			histptr = rinfo->history;
 			while(histptr != NULL) {
 			  pcl = histptr->update.tv_sec;
@@ -1079,7 +1082,11 @@ int output_routesum(R_LIST *rlist, net_ulong neighbor, FILE *ofp)
 #ifdef RTHISTORY
 		/*** Output Update History ***/
 		if ((rinfo->history != NULL) && (detail == true)){
-			fprintf(ofp, "    == Prefix Update History ==\n");
+			pcl = rinfo->firstupdate.tv_sec;
+			tm = localtime(&pcl);
+			strftime(prline, 128,
+	 "    == Prefix Update History (First %Y/%m/%d %H:%M:%S) ==\n", tm);
+			fprintf(ofp, prline);
 			histptr = rinfo->history;
 			while(histptr != NULL) {
 			  pcl = histptr->update.tv_sec;
@@ -1286,7 +1293,7 @@ int bvc_showipbgproute(BVS_CMD_PARAM *prm)
 	clno = (prm->adrs).clno;
 	prefix = (prm->adrs).addr;
 
-	bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+	bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 	bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 
@@ -1430,7 +1437,7 @@ int bvc_showipbgproutetime_eq(BVS_CMD_PARAM *prm)
 
 	/***** Print Route *****/
 	if (*(cnode+clno) == NULL) {
-		bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+		bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 		bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 	}
@@ -1500,7 +1507,7 @@ int bvc_showipbgproutetime_le(BVS_CMD_PARAM *prm)
 
 	/***** Print Route *****/
 	if (*(cnode+clno) == NULL) {
-		bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+		bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 		bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 	}
@@ -1570,7 +1577,7 @@ int bvc_showipbgproutetime_ge(BVS_CMD_PARAM *prm)
 
 	/***** Print Route *****/
 	if (*(cnode+clno) == NULL) {
-		bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+		bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 		bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 	}
@@ -1875,6 +1882,125 @@ int bvc_showcronlist(BVS_CMD_PARAM *prm)
 	return(BVS_CNT_END);
 }
 
+int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	static RADIX_T	**cnode;
+	static int	*upcode;
+	static NEXTHOP_LIST	**hoplist;
+	RADIX_T		*node;
+	char		prline[128];
+	int		cntr;
+	NEXTHOP_LIST	*listtmp, *ftmp;
+	R_LIST		*cptr;
+	ROUTE_INFO	*routeinfo;
+	int		allroutes;
+	struct in_addr	inaddr;
+
+	if (prm == NULL) {
+		cnode = (RADIX_T **)malloc(sizeof(RADIX_T *) * bvs_maxcon);
+		bzero(cnode, sizeof(RADIX_T *) * bvs_maxcon);
+		upcode = (int *)malloc(sizeof(int) * bvs_maxcon);
+		bzero(upcode, sizeof(int) * bvs_maxcon);
+		hoplist = (NEXTHOP_LIST **)malloc(sizeof(NEXTHOP_LIST *) * 
+								bvs_maxcon);
+		bzero(hoplist, sizeof(NEXTHOP_LIST *) * bvs_maxcon);
+		return(0);
+	}
+
+	clno = (prm->noparam).clno;
+
+	if (*(cnode+clno) == NULL) {
+		bvs_write(clno, "BGP Next Hop Summarizing ...");
+		*(hoplist + clno) = 0;
+	}
+
+	node = show_route_all(*(cnode+clno), (upcode+clno));
+	if (node == NULL) {
+		if (*(cnode+clno) == NULL) {
+			bvs_write(clno, "\r\n%% No Route\r\n");
+		}
+		else {
+			bvs_write(clno, "\r\nNext Hop  Summary ------\r\n");
+			sprintf(prline, "  Received Prefixes : %ld\r\n",
+								radix_prefixes());
+			bvs_write(clno, prline);
+			allroutes = radix_routes();
+			sprintf(prline, "  Received Routes   : %ld\r\n",
+								 allroutes);
+			bvs_write(clno, prline);
+			sprintf(prline, "  Received Nodes    : %ld\r\n", 
+								radix_nodes());
+			bvs_write(clno, prline);
+
+			bvs_write(clno, "\r\nNext Hop        Routes Rate\r\n");
+			listtmp = *(hoplist+clno);
+			while(listtmp != NULL) {
+				inaddr.s_addr = listtmp->next_hop;
+				sprintf(prline, "%15s %6d %5.1f%%\r\n",
+					inet_ntoa(inaddr), listtmp->count,
+					((float)(listtmp->count)/(float)allroutes)*100);
+				bvs_write(clno, prline);
+				listtmp = listtmp->next;
+			}
+			bvs_write(clno, "\r\n");
+			listtmp = *(hoplist+clno);
+			while(listtmp != NULL) {
+				ftmp = listtmp->next;
+				free(listtmp);
+				listtmp = ftmp;
+			}
+
+		}
+	} else {
+		cptr = node->rinfo;
+		while(cptr != NULL) {
+			routeinfo = cptr->route_data;
+			if ((*(hoplist+clno)) == NULL) {
+				*(hoplist+clno) = (NEXTHOP_LIST *)malloc(
+						  sizeof(NEXTHOP_LIST));
+				bzero(*(hoplist+clno), sizeof(NEXTHOP_LIST));
+				(*(hoplist+clno))->next_hop = 
+							routeinfo->next_hop;
+				(*(hoplist+clno))->count = 1;
+			} else {
+				listtmp = *(hoplist + clno);
+				while(1) {
+					if (listtmp->next_hop == 
+							routeinfo->next_hop) {
+						listtmp->count++;
+						break;
+					}
+					if (listtmp == NULL) {
+						listtmp->next = 
+							(NEXTHOP_LIST *)malloc(
+	                                                  sizeof(NEXTHOP_LIST));
+						bzero(listtmp->next, 
+							sizeof(NEXTHOP_LIST));
+						(listtmp->next)->next_hop = 
+							routeinfo->next_hop;
+						(listtmp->next)->count = 1;
+						break;
+					}
+					listtmp = listtmp->next;
+				}
+			}
+			cptr = cptr->next;
+		}
+	}
+	if (node == NULL) {
+		*(cnode+clno) = NULL;
+		if (*(cnode+clno) == NULL) {
+			return(BVS_CNT_END);
+		} else {
+			return(BVS_CNT_FINISH);
+		}
+	} else {
+		*(cnode+clno) = node;
+		return(BVS_CNT_CONTINUE);
+	}
+}
+
 #ifdef DEBUG
 
 #ifdef RADIX
