@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell 			                        */
 /****************************************************************/
-/* bvshell.c,v 1.42 2001/04/03 09:25:01 kuniaki Exp */
+/* bvshell.c,v 1.44 2001/07/30 08:53:49 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 Internet Initiative Japan Inc.
@@ -171,6 +171,7 @@ void bvs_init_set_default()
 	bvc_showipbgproutetime_le((BVS_CMD_PARAM *)0);
 	bvc_showipbgproutetime_ge((BVS_CMD_PARAM *)0);
 	bvc_clear_route_history((BVS_CMD_PARAM *)0);
+	bvc_show_bgpnexthop_sum((BVS_CMD_PARAM *)0);
 
 	/* Entry Commands */
 	bvs_entry_command(bvs_cmd_exit, "quit", "Logout BGPView Shell", false);
@@ -280,6 +281,9 @@ void bvs_init_set_default()
 	bvs_entry_command(bvc_showcronlist,
 			"show cron list", 
 			"Show Configurated Cron List", false);
+	bvs_entry_command(bvc_show_bgpnexthop_sum,
+			"show ip nexthop summary", 
+			"Show BGP Next Hop Summary", false);
 
 #ifdef DEBUG
 #ifdef RADIX
@@ -1183,8 +1187,11 @@ int ins_char(int clno, const char *ch, int cnt, int mode)
 		if ((bvs_clients+clno)->status == BVSC_LOGIN) break;
 		strcpy(buf, (((bvs_clients+clno)->cmdline)+curcol));
 		bzero(buf2, MAXIMUM_CMD_LENGTH);
+		memset(buf2, ' ', cnt);
+		strcat(buf, buf2);
+		bzero(buf2, MAXIMUM_CMD_LENGTH);
 		memset(buf2, '\b', 
-				strlen((((bvs_clients+clno)->cmdline)+curcol)));
+			strlen((((bvs_clients+clno)->cmdline)+curcol))+cnt);
 		strcat(buf, buf2);
 		bvs_write(clno, buf);
 		break;
@@ -1553,17 +1560,24 @@ int bvs_input(fd_set *fds)
 					cpnt++;
 					break;
 				case 0x01:
-					/* Move Cur Top */
+					/* Move Cur Top (Ctrl-a) */
 					mov_cur(cnt, 2, (bvs_clients+cnt)->col);
 					cpnt++;
 					break;
 				case 0x04:
-					/* Delete Char */
+					/* Delete Char (Ctrl-d) */
 					ins_char(cnt, NULL, 1, BVSE_DEL);
 					cpnt++;
 					break;
+				case 0x0B:
+					/* Clear from cursor (Ctrl-k) */
+					ins_char(cnt, NULL,
+					strlen((bvs_clients+cnt)->cmdline) -
+					(bvs_clients+cnt)->col, BVSE_DEL);
+					cpnt++;
+					break;
 				case 0x0C:
-					/* Clear Screen */
+					/* Clear Screen (Ctrl-l) */
 					clearscreen(cnt);
 					cpnt++;
 					break;
