@@ -4,7 +4,7 @@
 /*                                                              */
 /* Radix Tree Routine                                           */
 /****************************************************************/
-/* radix.c,v 1.17 2001/04/02 15:09:06 kuniaki Exp */
+/* radix.c,v 1.18 2001/04/19 11:42:57 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -77,6 +77,7 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 {
 	ROUTE_INFO	*ribuf;
 	R_LIST		*rlist,*brlist,*lbuf;
+	struct timeval	tmvtemp;
 #ifdef RTHISTORY
 	RT_HISTORY	*histbuf, *histendbuf, *hhist;
 	boolean		actbuf;
@@ -118,6 +119,7 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 			histbuf = (rlist->route_data)->history;
 			histendbuf = (rlist->route_data)->histend;
 			actbuf = (rlist->route_data)->active;
+			tmvtemp.tv_sec = (rlist->route_data)->firstupdate.tv_sec;
 #endif
 			free(rlist->route_data);
 			rlist->route_data = ribuf;
@@ -125,6 +127,8 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 			(rlist->route_data)->history = histbuf;
 			(rlist->route_data)->histend = histendbuf;
 			(rlist->route_data)->active = true;
+			(rlist->route_data)->firstupdate.tv_sec = tmvtemp.tv_sec;
+			(rlist->route_data)->firstupdate.tv_usec = 0;
 
 			hhist = (RT_HISTORY *)malloc(sizeof(RT_HISTORY));
 			if (hhist == NULL) {
